RealTimeEvent = {}
local RealTimeEvent_mt = Class(RealTimeEvent, Event)

InitEventClass(RealTimeEvent, "RealTimeEvent")

function RealTimeEvent.emptyNew()
	return Event.new(RealTimeEvent_mt)
end
---@class RealTimeEvent
function RealTimeEvent.new(valueType, valueName, valueData)
	local self = RealTimeEvent.emptyNew()
    --printError("RealTimeEvent::new called")
    --printError("valueType:"..tostring(valueType))
    --printError("valueName:"..tostring(valueName))
    --printError("valueData:"..tostring(valueData))
  self.valueType = valueType
	self.valueName = valueName
  self.valueData = valueData
	return self
end

function RealTimeEvent:readStream(streamId, connection)
    --printError("readStream event")
  self.valueType = streamReadUInt8(streamId)
  self.valueName = streamReadString(streamId)
  -- Check if valuetype is nil
  if self.valueType == nil then 
    printError(g_currentModName..':RealTimeEvent:readStream:Error:valueType=nil - valueType can not be nil.  Must be int.')
    return
  end
  -- Check if valueName is nil
  if self.valueName == nil then 
    printError(g_currentModName..':RealTimeEvent:readStream:Error:valueName=nil - valueName can not be nil.  Must be string.')
    return
  end
  -- String
  if     self.valueType == 1 then self.valueData = streamReadString(streamId) 
  -- Bool
  elseif self.valueType == 2 then self.valueData = streamReadBool(streamId) 
  -- Int8
  elseif self.valueType == 3 then self.valueData = streamReadInt8(streamId) 
  -- Int32
  elseif self.valueType == 4 then self.valueData = streamReadInt32(streamId) 
  -- UInt8
  elseif self.valueType == 5 then self.valueData = streamReadUInt8(streamId) 
  -- UInt16
  elseif self.valueType == 6 then self.valueData = streamReadUInt16(streamId) 
  -- UIntN
  elseif self.valueType == 7 then self.valueData = streamReadUIntN(streamId) 
  -- Float32
  elseif self.valueType == 8 then self.valueData = streamReadFloat32(streamId)
  -- Unknown
  else printError(g_currentModName..':RealTimeEvent:readStream:Error:valueType: %s not correct type.') end
  -- Debug
	self:run(connection)
end

function RealTimeEvent:writeStream(streamId, connection)
    --printError("writestream event")
    --printError("valueType:"..tostring(self.valueType))
    --printError("valueName:"..tostring(self.valueName))
    --printError("valueData:"..tostring(self.valueData))
  streamWriteUInt8(streamId, self.valueType)
  streamWriteString(streamId, self.valueName)
  -- Check if valuetype is nil
  if self.valueType == nil then 
    return
  end
  -- Check if valueName is nil
  if self.valueName == nil then 
    return
  end
  -- String
  if     self.valueType == 1 then streamWriteString(streamId, self.valueData) 
  -- Bool
  elseif self.valueType == 2 then streamWriteBool(streamId, self.valueData) 
  -- Int8
  elseif self.valueType == 3 then streamWriteInt8(streamId, self.valueData) 
  -- Int32
  elseif self.valueType == 4 then streamWriteInt32(streamId, self.valueData)
  -- UInt8
  elseif self.valueType == 5 then streamWriteUInt8(streamId, self.valueData) 
  -- UInt16
  elseif self.valueType == 6 then streamWriteUInt16(streamId, self.valueData) 
  -- UIntN
  elseif self.valueType == 7 then streamWriteUIntN(streamId, self.valueData) 
  -- Float32
  elseif self.valueType == 8 then streamWriteFloat32(streamId, self.valueData)
  -- Unknown
  else 
    streamWriteInt8(streamId, 0)
  end

  
end

function RealTimeEvent:run(connection)
    local permission=false
    if not connection:getIsServer() then
        --printError("run on event server")
        local player=g_currentMission:getPlayerByConnection(connection)
        local user = g_currentMission.userManager:getUserByUserId(player.userId)
        if user:getIsMasterUser() then
            permission=true
            g_server:broadcastEvent(RealTimeEvent.new(self.valueType, self.valueName, self.valueData), nil, connection)
        else
            g_server:broadcastEvent(RealTimeEvent.new(1, "realTimeSyncNoti", "changing settings is only allowed by admins"), nil, nil)
        end
    --g_server:broadcastEvent - data being sent from client to server")
		
    end
    if g_WsrealTimeSync ~= nil then
        --printError("g_WsrealTimeSync is not nil")
        if self.valueName ~= nil and self.valueName == "realTimeSyncNoti" then
            -- Display notification to player
            --printError("add Ingame Notification event")
            g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_INFO, self.valueData)
        else
            if permission==true then
                -- Save realtime data from server to local
            g_WsrealTimeSync.settings:setValue(self.valueName, self.valueData)
            g_WsrealTimeSync.settings:saveSettings()
            end
            
        end
    else
        --printError("g_WsrealTimeSync is nil")
    end
end

function RealTimeEvent.sendEvent(valueType, valueName, valueData)
    --printWarning("RealTimeEvent.sendEvent")
    --printError("valueType:"..tostring(valueType))
    --printError("valueName:"..tostring(valueName))
    --printError("valueData:"..tostring(valueData))
	if g_server ~= nil then
        --printWarning("RealTimeEvent.sendEvent server")
		g_server:broadcastEvent(RealTimeEvent.new(valueType, valueName, valueData))
	else
        --printWarning("RealTimeEvent.sendEvent client")
		g_client:getServerConnection():sendEvent(RealTimeEvent.new(valueType, valueName, valueData))
	end
end