RealTimeWorker = {}

local RealTimeWorker_mt = Class(RealTimeWorker)

function RealTimeWorker:new(mission, modDirectory, modName, logger)
    local self = setmetatable({}, RealTimeWorker_mt)
    self.logger            = logger
    self.logger:printVariable(self.logger,"RealTimeWorker:new.logger:",WsLogger.Severity.Debug,1)
    self.isServer          = mission:getIsServer()
    self.logger:printVariable(self.isServer,"RealTimeWorker:new.isServer:",WsLogger.Severity.Debug,1)
    self.isClient          = mission:getIsClient()
    self.logger:printVariable(self.isClient,"RealTimeWorker:new.isClient:",WsLogger.Severity.Debug,1)
    self.isMPGame          = g_currentMission.missionDynamicInfo.isMultiplayer
    self.logger:printVariable(self.isMPGame,"RealTimeWorker:new.isMPGame:",WsLogger.Severity.Debug,1)
    self.mission           = mission
    --self.logger:printVariable(self.mission,"RealTimeWorker:new.mission:",WsLogger.Severity.Debug,0)
    self.modDirectory      = modDirectory
    self.logger:printVariable(self.modDirectory,"RealTimeWorker:new.modDirectory:",WsLogger.Severity.Debug,1)
    
    self.gameInfoDisplay   = mission.hud.gameInfoDisplay
    self.logger:printVariable(self.gameInfoDisplay,"RealTimeWorker:new.gameInfoDisplay:",WsLogger.Severity.Debug,0)
    self.justSetFalse      = false
    self.isServer=mission:getIsServer()
    self.logger:printVariable(self.isServer,"RealTimeWorker:new.isServer:",WsLogger.Severity.Debug,1)
    source(modDirectory..'scripts/common/WsSettings.lua')

    self.settings=WsSettings:new(modName,'RealTimeSyncSettings.xml', true, nil, nil,
    {
      enableTimeSync=false,
      serverOffsetUTC={-5,"int"},
    },
    logger
  
  )

    self.debugTimerRuns         = 0
    self.setValueTimerFrequency = 60
    self.inspectText            = {}
    self.menuTextSizes          = { 8, 10, 12, 14, 16 }

    local modDesc       = loadXMLFile("modDesc", modDirectory .. "modDesc.xml");
    self.version        = getXMLString(modDesc, "modDesc.version");
    self.logger:printVariable(self.version,"RealTimeWorker:new.version:",WsLogger.Severity.Debug,1)
    delete(modDesc)

    self.logger:print(":RealTimeWorker:new() Initialized",WsLogger.Severity.Verbose)

    self.ChangedScale = false
    self.ServerHour = 0
    self.ServerMinCleanUTC = 0
    self.TimeSpeed = 1
    
  return self

end
function RealTimeWorker:update(dt)
  local TimeDifff=0
  local TimeSpeed = 1
  self.logger:print("update called",WsLogger.Severity.Call)
  if g_updateLoopIndex % self.setValueTimerFrequency == 0 and RealTimeWorker:singleServerCheck() then
    self.logger:print("it's time for time check",WsLogger.Severity.Debug)
    self.settings:loadSettings()
    if self.settings:getValue("enableTimeSync") == false then
      -- if just set to false, we need to set timescale
      if self.justSetFalse == true then
          RealTimeWorker:setTimeScale(1)
          self.justSetFalse = false
      end
      return
    else
      self.justSetFalse = true
    end
    self.logger:print("check ok",WsLogger.Severity.Debug)
    local m_serverUnixTime=getTime()
    local m_servertotalMinutes=math.floor(m_serverUnixTime/60)
    local m_serverhour=math.floor((m_servertotalMinutes/60)%24)
    local m_serverminute=m_servertotalMinutes%60
    self.logger:print("Stunde:"..m_serverhour,WsLogger.Severity.Debug)
    self.logger:print("Minute:"..m_serverminute,WsLogger.Severity.Debug)
    self.logger:print("Time:"..m_serverUnixTime,WsLogger.Severity.Debug)
    local ServerTime = m_serverhour*100 + m_serverminute
    self.logger:print("Server Time original:"..ServerTime,WsLogger.Severity.Debug)
    local setting_serverOffsetUTC=self.settings:getValue("serverOffsetUTC")
    local setting_enableTimeSync=self.settings:getValue("enableTimeSync")
    if type(setting_serverOffsetUTC)=="number" and setting_serverOffsetUTC~=0 then
      m_serverhour=m_serverhour+setting_serverOffsetUTC
      SollTime=m_serverhour*100 + m_serverminute
      self.logger:printVariable(SollTime,"Server Time Corrected (Solltime):",WsLogger.Severity.Debug)
    end
    if SollTime>2400 then
      SollTime=SollTime-2400
    elseif ServerTime<0 then
      SollTime=SollTime+2400
    end
    local GameTime=g_currentMission.environment.currentHour*100+g_currentMission.environment.currentMinute
    self.logger:printVariable(GameTime,"Game Time:",WsLogger.Severity.Debug)
    local TimeDifff=SollTime-GameTime
    self.logger:printVariable(TimeDifff,"Time Diff:",WsLogger.Severity.Debug)
    if TimeDifff<-1200 then
      TimeDifff=TimeDifff+2400
    elseif TimeDifff>1200 then
      TimeDifff=TimeDifff-2400
    end
    self.ChangedScale=false
    local shouldScale=g_currentMission.missionInfo.timeScale
    self.logger:printVariable(TimeDifff,"Time Diff Corrected:",WsLogger.Severity.Debug)
    self.logger:printVariable(g_currentMission.missionInfo.timeScale,"Actual timescale",WsLogger.Severity.Debug)
    if TimeDifff > -6 and TimeDifff < 6 and g_currentMission.missionInfo.timeScale == 1 then
        -- we are at timescale 1 and timediff is <=5 (negative or postive), so we don't need to change timescale
      shouldScale=1
    elseif TimeDifff < 0 and g_currentMission.missionInfo.timeScale>1 then
        --we need to stop fast timescale
      shouldScale=1
    elseif TimeDifff > 0 and g_currentMission.missionInfo.timeScale<1 then
      --we need to stop slow timescale
      shouldScale=1
    elseif TimeDifff > 600 then
        -- we need to increase timescale fast
      shouldScale=360
    elseif TimeDifff > 50 then
        -- we need to increase timescale medium
      shouldScale=30
    elseif TimeDifff > 5 then
        --we need to increase timescale slow
      shouldScale=5
    elseif TimeDifff < -100 then
        -- we need to descrease timescale fast
      shouldScale=0
    elseif TimeDifff < -30 then
        -- we need to descrease timescale medium
      shouldScale=0.1
    elseif TimeDifff <-10 then
        --we need to descrease timescale slow
      shouldScale=0.5
    end
    if shouldScale~=g_currentMission.missionInfo.timeScale then
      RealTimeWorker:setTimeScale(shouldScale)
      local notifText=""
      if shouldScale~= 1 then
        notifText="Time Synchronisation in Progress, target Time:".. math.floor(SollTime/100)..":"..string.format("%02d",(SollTime%100))
      else
        notifText="Time Synchronisation finished"
      end
      self.logger:print(notifText,WsLogger.Severity.Verbose)
      if self.isServer then
        self.logger:print("SendBroadcast",WsLogger.Severity.Verbose)
        if g_server==nil then
          --printError("Server nil")
        else
          --printWarning("Server available")
        end
        local ev=RealTimeEvent.new(1, "realTimeSyncNoti", notifText)
        if RealTimeWorker:isSinglePlayer() then
          g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_INFO, notifText)
        end
        g_server:broadcastEvent(ev, nil, nil)
      else
        self.logger:print("IngameNotification",WsLogger.Severity.Verbose)
        --g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_INFO, notifText)
        g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_INFO, notifText)
      end
    end
  end
end
function RealTimeWorker:setTimeScale(scale)
    if RealTimeWorker:isSinglePlayer() then
        g_currentMission.missionInfo.timeScale=scale
    else
        g_currentMission:setTimeScale(scale)
    end
end
function RealTimeWorker:isSinglePlayer()
  if g_currentMission:getIsServer() == true and g_currentMission:getIsClient() == true and g_dedicatedServer == nil and g_server ~= nil then
    return true
  else
    return false
  end
end
-- Check if client is a server or single player
function RealTimeWorker:singleServerCheck()
  -- Check if server
  if g_currentMission:getIsServer() == true and g_currentMission:getIsClient() == true and g_dedicatedServer ~= nil and g_server ~= nil then
    -- Multiplayer Server Host
    return true
  -- Check if single player
  elseif g_currentMission:getIsServer() == true and g_currentMission:getIsClient() == true and g_dedicatedServer == nil and g_server ~= nil then
    -- Single player client
    return true
  else
    -- Client on dedicated server
    return false
  end
end
function RealTimeWorker:save()
  self.logger:print("save called",WsLogger.Severity.Call)
end
function RealTimeWorker:onStartMission(mission)
    self.logger:print("on Start Mission",WsLogger.Severity.Call)
    if not self.isClient then
        return
    end
    self.logger:print("start mission client",WsLogger.Severity.Call)
    self.settings:loadSettings()
	self.settings:saveSettings()
end
function RealTimeWorker:enable()
  self.logger:print("RealTimeWorker:enable called",WsLogger.Severity.Call)
  self.settings:setValue("enableTimeSync",true)
  self.settings:saveSettings()
  if not self.isServer then
    RealTimeEvent.sendEvent(2,"enableTimeSync",true)
  end
  self.logger:print("RealTimeWorker enabled",WsLogger.Severity.Warning)
end
function RealTimeWorker:disable()
  self.logger:print("RealTimeWorker:disable called",WsLogger.Severity.Call)
  self.settings:setValue("enableTimeSync",false)
  self.settings:saveSettings()
  if not self.isServer then
    RealTimeEvent.sendEvent(2,"enableTimeSync",false)
  end
  self.logger:print("RealTimeWorker disabled",WsLogger.Severity.Warning)
end
function RealTimeWorker:setoffset(p_offset)
  self.logger:print("RealTimeWorker:setoffset called",WsLogger.Severity.Call)
  local m_offset=tonumber(p_offset)
  if m_offset~=nil then
    self.settings:setValue("serverOffsetUTC",m_offset)
    self.settings:saveSettings()
    if not self.isServer then
      RealTimeEvent.sendEvent(4,"serverOffsetUTC",m_offset)
    end
    self.logger:printVariable(m_offset,"RealTimeWorker offset set to ",WsLogger.Severity.Warning)
  else
    self.logger:print("RealTimeWorker:setoffset provided offset is not a number, please provide a number as parameter", WsLogger.Severity.Error)
  end
end
function RealTimeWorker:canSleep()
  return false
end